from .Tools.check_system import is_it_mac
from .Tools.check_venv import is_on_venv
from .beeware_tools.edit_beeware_project import edit_an_exist_project
import sys
import sys

# python -m fpyo2ipa.build
args = str(sys.argv)

if is_on_venv() == False:
    #? exit if not in venv.
    sys.exit("Exit: Please run this on a virtual environment. Run this command to create one: `python3 -m venv venv`")


#! important informations
print("""
This is `fpyo2apk`. A flet-pyodide dist to apk converter.
Be sure that Android Studio is installed and setuped.
Starting..

""")

#! Step1: Create the project
edit_an_exist_project()


The_app_py_script = """

import toga
from toga.style import Pack
from toga.style.pack import COLUMN, ROW
import time
import webbrowser
from .localhost import LocalHoat


# basedurlhere
class AppNameBabe(toga.App):

    def startup(self):
        main_box = toga.Box(style=Pack(direction=COLUMN))

        #? Get and setup the localhost
        dist_folder_path = str(__file__).replace("app.py", "assets/dist")
        LH = LocalHoat()
        LH.start()
        for i in range(15):
            if LH.started == True: break
            time.sleep(0.5)
        
        #! ---
        import shutil
        import os
        sandbox_documents_dir = os.path.expanduser('~/Documents')
        
        if os.path.isdir(f"{sandbox_documents_dir}/dist"):
            shutil.rmtree(f"{sandbox_documents_dir}/dist")
        shutil.copytree(dist_folder_path, f"{sandbox_documents_dir}/dist")

        the_file = open(f"{sandbox_documents_dir}/dist/index.html", encoding="utf-8").read()
        the_file = the_file.replace("/basedurlhere/", f"{sandbox_documents_dir}/dist/")
        open(f"{sandbox_documents_dir}/dist/index.html", "w+", encoding="utf-8").write(the_file)
        
        #! ---
        
        #? reWrite the index file.
        if 1 != 1:
            the_file = open(f"{dist_folder_path}/index.html", encoding="utf-8").read()
            the_file = the_file.replace("/basedurlhere/", f"{dist_folder_path}/")
            open(f"{dist_folder_path}/index.html", "w+", encoding="utf-8").write(the_file)

        #? create the webview
        webview = toga.WebView(style=Pack(flex=1))
        url = f'http://localhost:{LH.PORT}/{sandbox_documents_dir}/dist'
        webview.url = url
        main_box.add(webview)

        self.main_window = toga.MainWindow(title=self.formal_name)
        self.main_window.content=main_box
        self.main_window.show()


def main():
    return AppNameBabe()


"""


The_localhost_py_script = """
import threading
import subprocess

class LocalHoat :
    def __init__(self) -> None:
        import socketserver

        self.started = False
        with socketserver.TCPServer(("localhost", 0), None) as s:
            free_port = s.server_address[1]
            self.PORT = int(free_port)
    
    def start (self):
        threading.Thread(target=self.run_server, daemon=True).start()
    
    def run_server (self):
        import http.server
        import socketserver
        import threading
        import webbrowser
        import time
        import random
            
        PORT = self.PORT # Change this to the desired port number
            
        class MyHandler(http.server.SimpleHTTPRequestHandler):
            def log_message(self, format, *args):
                pass
            
        with socketserver.TCPServer(("", PORT), MyHandler) as httpd:
            self.started = True
            httpd.serve_forever()
"""


import os
import zipfile
import shutil

def Unzip_asset ():
    asset_path = str(__file__).replace("Tools/unzip_assets.py", "assets/fpyo2apkdist.zip")
    if not os.path.isfile(asset_path):
        raise FileNotFoundError("Cannot found the assets.")
    
    if os.path.isdir("fpyo2apkdist"):
        shutil.rmtree("fpyo2apkdist")

    with zipfile.ZipFile(asset_path, "r") as zip_ref:
        zip_ref.extractall("fpyo2apkdist")
    
    shutil.copytree("fpyo2apkdist/fpyo2apkdist", "_fpyo2apkdist")
    shutil.rmtree("fpyo2apkdist")
    shutil.move("_fpyo2apkdist", "fpyo2apkdist")

    #? Edit the pyproject.toml file.
    AppName = input("Write the name of your app: ")
    Describe = input("Write a small descibe of your app:")
    if str(AppName).replace(" ", "") == "": AppName = "fpyo2apk"
    if '"' in str(AppName): AppName = "fpyo2apk"
    if '"' in str(Describe): Describe = "fpyo2apk"

    print("setting your appname and describe..")
    read_file = open("fpyo2apkdist/pyproject.toml", encoding="utf-8").read()
    read_file = str(read_file).replace("_here_the_app_name_", AppName)
    read_file = str(read_file).replace("_here_the_describe_of_app_", Describe)

    open("fpyo2apkdist/pyproject.toml", "w+", encoding="utf-8").write(read_file)

    print("unzip is done..")


from .the_app_py import The_app_py_script
from .the_localhost_py import The_localhost_py_script
from ..Tools.unzip_assets import Unzip_asset
import os
import shutil
import subprocess
import sys

def edit_an_exist_project():
    print("Create a briefcase project..")
    Unzip_asset()

    
    project_name = "fpyo2apkdist"
    if not os.path.isdir(project_name):
        raise FileNotFoundError(f"There is no folder with path '{project_name}'.")
    
    if not os.path.isdir(f"{project_name}/src"):
        raise FileNotFoundError(f"There is no folder with path '{project_name}/src'.")
    
    if not os.path.isdir(f"{project_name}/src/fpyo2apk/assets"):
        os.mkdir(f"{project_name}/src/fpyo2apk/assets")
    
    if not os.path.isdir("dist"):
        raise FileNotFoundError("There must be a flet-pyodide folder called `dist` to start.")
    
    print("editing the dist..")
    if '  <base href="/basedurlhere/">' in open("dist/index.html", encoding="utf-8").read():
        pass
    elif '  <base href="/">' not in open("dist/index.html", encoding="utf-8").read():
        sys.exit("Exit with Error: The base/href url of the dist must be '/'.")
    
    the_index_file = open("dist/index.html", encoding="utf-8").read()
    the_index_file = str(the_index_file).replace('  <base href="/">', '  <base href="/basedurlhere/">')
    open("dist/index.html", "w+", encoding="utf-8").write(the_index_file)

    print("copy the dist..")
    if os.path.isdir(f"{project_name}/src/fpyo2apk/assets/dist"):
        shutil.rmtree(f"{project_name}/src/fpyo2apk/assets/dist")
    shutil.copytree("dist", f"{project_name}/src/fpyo2apk/assets/dist")

    print("start edit the 'app.py' file")
    open(f"{project_name}/src/fpyo2apk/app.py", "w+", encoding="utf-8").write(The_app_py_script)

    print("start creating/editing the 'localhost.py' file")
    open(f"{project_name}/src/fpyo2apk/localhost.py", "w+", encoding="utf-8").write(The_localhost_py_script)

    print("Your app is being built and then can open it on the android simulator..\nThis Usually take two minutes to finish, please wait..")
    process = subprocess.Popen(["cd fpyo2apkdist\nbriefcase create Android\nbriefcase build Android\n"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)
    output, error = process.communicate()
    if error != None:
        sys.exit(f"\nExit with Error: {error}")
    
    print("""

All work done!
Your app is in the `fpyo2apkdist/build/fpyo2apkdist/android/`!
To run a simulator, use:
$ cd fpyo2apkdist
$ briefcase run Android

    """)


from setuptools import setup, find_packages

setup(
    name='fpyo2apk',
    version='1.1.3',
    author='SKbarbon',
    description='A package tool that allow you to built a python Android apps with flet-pyodide dist.',
    long_description="https://github.com/SKbarbon/fpyo2apk",
    url='https://github.com/SKbarbon/fpyo2apk',
    install_requires=["briefcase==0.3.14"],
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows"
    ],
    include_dirs=["assets"],
    package_data={"assets": ["pyo2ipadist.zip"]},
)